#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;
layout (location = 3) in vec3 tangent;

#define NR_POINT_LIGHTS 1

struct PointLight {
	vec3 position;

	float constant;
	float linear;
	float quadratic;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

out VS_OUT {
	vec3 FragPos;
    vec2 TexCoords;
    vec3 TangentLightPos[NR_POINT_LIGHTS];
    vec3 TangentViewPos;
    vec3 TangentFragPos;
} vs_out;

uniform mat4 projection;
uniform mat4 view;
uniform mat4 model;
uniform vec3 viewPos;

uniform PointLight pointLights[NR_POINT_LIGHTS];

void main()
{
    gl_Position = projection * view * model * vec4(position, 1.0f);
    vs_out.FragPos = vec3(model * vec4(position, 1.0));   
    vs_out.TexCoords = texCoords;
    
	// normal
	vec3 n = normalize(vec3(model * vec4(normal, 0.0f)));
	// tangent
	vec3 t = normalize(vec3(model * vec4(tangent, 0.0f)));
	t = normalize(t - dot(t,n) * n);
	// bitangent
	vec3 b = cross(t,n);
    
	//TBN matrix
    mat3 TBN = transpose(mat3(t, b, n));  

	// multiply every position with TBN matrix
	for (int i = 0; i < NR_POINT_LIGHTS; i++) {
		vs_out.TangentLightPos[i] = TBN * pointLights[i].position;
	}
    vs_out.TangentViewPos  = TBN * viewPos;
    vs_out.TangentFragPos  = TBN * vs_out.FragPos;
}